@@ -66,13 +66,39 @@ struct pw_link_proxy;
 
 #define PW_VERSION_CORE				0
 
-#define PW_CORE_PROXY_METHOD_UPDATE_TYPES	0
-#define PW_CORE_PROXY_METHOD_SYNC		1
-#define PW_CORE_PROXY_METHOD_GET_REGISTRY	2
-#define PW_CORE_PROXY_METHOD_CLIENT_UPDATE	3
-#define PW_CORE_PROXY_METHOD_CREATE_OBJECT	4
-#define PW_CORE_PROXY_METHOD_CREATE_LINK	5
-#define PW_CORE_PROXY_METHOD_NUM		6
+#define PW_CORE_PROXY_METHOD_HELLO		0
+#define PW_CORE_PROXY_METHOD_UPDATE_TYPES	1
+#define PW_CORE_PROXY_METHOD_SYNC		2
+#define PW_CORE_PROXY_METHOD_GET_REGISTRY	3
+#define PW_CORE_PROXY_METHOD_CLIENT_UPDATE	4
+#define PW_CORE_PROXY_METHOD_PERMISSIONS	5
+#define PW_CORE_PROXY_METHOD_CREATE_OBJECT	6
+#define PW_CORE_PROXY_METHOD_NUM		7
+
+/**
+ * Key to update default permissions of globals without specific
+ * permissions. value is "[r][w][x]" */
+#define PW_CORE_PROXY_PERMISSIONS_DEFAULT	"permissions.default"
+
+/**
+ * Key to update specific permissions of a global. If the global
+ * did not have specific permissions, it will first be assigned
+ * the default permissions before it is updated.
+ * Value is "<global-id>:[r][w][x]"*/
+#define PW_CORE_PROXY_PERMISSIONS_GLOBAL	"permissions.global"
+
+/**
+ * Key to update specific permissions of all existing globals.
+ * This is equivalent to using \ref PW_CORE_PROXY_PERMISSIONS_GLOBAL
+ * on each global id individually that did not have specific
+ * permissions.
+ * Value is "[r][w][x]" */
+#define PW_CORE_PROXY_PERMISSIONS_EXISTING	"permissions.existing"
+
+#define PW_LINK_OUTPUT_NODE_ID	"link.output_node.id"
+#define PW_LINK_OUTPUT_PORT_ID	"link.output_port.id"
+#define PW_LINK_INPUT_NODE_ID	"link.input_node.id"
+#define PW_LINK_INPUT_PORT_ID	"link.input_port.id"
 
 /**
  * \struct pw_core_proxy_methods
@@ -85,6 +111,11 @@ struct pw_link_proxy;
 struct pw_core_proxy_methods {
 #define PW_VERSION_CORE_PROXY_METHODS	0
 	uint32_t version;
+	/**
+	 * Start a conversation with the server. This will send
+	 * the core info and server types.
+	 */
+	void (*hello) (void *object);
 	/**
 	 * Update the type map
 	 *
@@ -122,6 +153,19 @@ struct pw_core_proxy_methods {
 	 * \param props the new client properties
 	 */
 	void (*client_update) (void *object, const struct spa_dict *props);
+	/**
+	 * Manage the permissions of the global objects
+	 *
+	 * Update the permissions of the global objects using the
+	 * dictionary with properties.
+	 *
+	 * Globals can use the default permissions or can have specific
+	 * permissions assigned to them.
+	 *
+	 * \param id the global id to change
+	 * \param props dictionary with permission properties
+	 */
+	void (*permissions) (void *object, const struct spa_dict *props);
 	/**
 	 * Create a new object on the PipeWire server from a factory.
 	 * Use a \a factory_name of "client-node" to create a
@@ -139,27 +183,14 @@ struct pw_core_proxy_methods {
 			       uint32_t version,
 			       const struct spa_dict *props,
 			       uint32_t new_id);
-	/**
-	 * Create a new link between two node ports
-	 *
-	 * \param output_node_id the global id of the output node
-	 * \param output_port_id the id of the output port
-	 * \param input_node_id the global id of the input node
-	 * \param input_port_id the id of the input port
-	 * \param filter an optional format filter
-	 * \param props optional properties
-	 * \param new_id the client proxy id
-	 */
-	void (*create_link) (void *object,
-			     uint32_t output_node_id,
-			     uint32_t output_port_id,
-			     uint32_t input_node_id,
-			     uint32_t input_port_id,
-			     const struct spa_pod *filter,
-			     const struct spa_dict *props,
-			     uint32_t new_id);
 };
 
+static inline void
+pw_core_proxy_hello(struct pw_core_proxy *core)
+{
+	pw_proxy_do((struct pw_proxy*)core, struct pw_core_proxy_methods, hello);
+}
+
 static inline void
 pw_core_proxy_update_types(struct pw_core_proxy *core, uint32_t first_id, const char **types, uint32_t n_types)
 {
@@ -186,6 +217,12 @@ pw_core_proxy_client_update(struct pw_core_proxy *core, const struct spa_dict *p
 	pw_proxy_do((struct pw_proxy*)core, struct pw_core_proxy_methods, client_update, props);
 }
 
+static inline void
+pw_core_proxy_permissions(struct pw_core_proxy *core, const struct spa_dict *props)
+{
+	pw_proxy_do((struct pw_proxy*)core, struct pw_core_proxy_methods, permissions, props);
+}
+
 static inline void *
 pw_core_proxy_create_object(struct pw_core_proxy *core,
 			    const char *factory_name,
@@ -200,24 +237,6 @@ pw_core_proxy_create_object(struct pw_core_proxy *core,
 	return p;
 }
 
-static inline struct pw_link_proxy *
-pw_core_proxy_create_link(struct pw_core_proxy *core,
-			  uint32_t type,
-			  uint32_t output_node_id,
-                          uint32_t output_port_id,
-                          uint32_t input_node_id,
-                          uint32_t input_port_id,
-                          const struct spa_pod *filter,
-                          const struct spa_dict *prop,
-			  size_t user_data_size)
-{
-	struct pw_proxy *p = pw_proxy_new((struct pw_proxy*)core, type, user_data_size);
-	pw_proxy_do((struct pw_proxy*)core, struct pw_core_proxy_methods, create_link, output_node_id, output_port_id,
-			input_node_id, input_port_id, filter, prop, pw_proxy_get_id(p));
-	return (struct pw_link_proxy*) p;
-}
-
-
 #define PW_CORE_PROXY_EVENT_UPDATE_TYPES 0
 #define PW_CORE_PROXY_EVENT_DONE         1
 #define PW_CORE_PROXY_EVENT_ERROR        2
@@ -328,8 +347,13 @@ pw_core_proxy_add_listener(struct pw_core_proxy *core,
  * request.  This creates a client-side proxy that lets the object
  * emit events to the client and lets the client invoke methods on
  * the object. See \ref page_proxy
+ *
+ * Clients can also change the permissions of the global objects that
+ * it can see. This is interesting when you want to configure a
+ * pipewire session before handing it to another application. You
+ * can, for example, hide certain existing or new objects or limit
+ * the access permissions on an object.
  */
-
 #define PW_REGISTRY_PROXY_METHOD_BIND		0
 #define PW_REGISTRY_PROXY_METHOD_NUM		1
 
@@ -383,9 +407,11 @@ struct pw_registry_proxy_events {
 	 * \param permissions the permissions of the object
 	 * \param type the type of the interface
 	 * \param version the version of the interface
+	 * \param props extra properties of the global
 	 */
 	void (*global) (void *object, uint32_t id, uint32_t parent_id,
-			uint32_t permissions, uint32_t type, uint32_t version);
+			uint32_t permissions, uint32_t type, uint32_t version,
+			const struct spa_dict *props);
 	/**
 	 * Notify of a global object removal
 	 *
