@@ -6,5 +6,6 @@ var structpw__protocol__client =
     [ "disconnect", "structpw__protocol__client.html#a4b60903a56e79e980b11ea2d14fac80c", null ],
     [ "link", "structpw__protocol__client.html#addd42578b59ddffc32a42a52754654be", null ],
     [ "protocol", "structpw__protocol__client.html#a2f6233dcd7a026275a93680ae68ee263", null ],
-    [ "remote", "structpw__protocol__client.html#a6879f664fbafbd1279e7f3cce224a25a", null ]
+    [ "remote", "structpw__protocol__client.html#a6879f664fbafbd1279e7f3cce224a25a", null ],
+    [ "steal_fd", "structpw__protocol__client.html#ae94059a4fa05da14961edb56be183f2a", null ]
 ];
\ No newline at end of file
